import re
from collections import Counter
from os.path import join, splitext

from rdkit import Chem
from rich.console import Console
from rich.table import Table

from csfdock.DVisualize import *
from csfdock.MolView import *
from csfdock.utils import get


console = Console()


class ProjectStart(MolView, DVisualize):
    """Creates a Project in Optimizing Scores and Docking
    Args:
        *args: Description
        **kwargs: Description
    """

    def __init__(self, *args, **kwargs):
        self.PROJECT_DIR = kwargs.get("path", os.getcwd())
        super().__init__(*args, **kwargs)
        self.AA = [
            "ALA",
            "ARG",
            "ASN",
            "ASP",
            "CYS",
            "GLN",
            "GLU",
            "GLY",
            "HIS",
            "ILE",
            "LEU",
            "LYS",
            "MET",
            "PHE",
            "PRO",
            "SER",
            "THR",
            "TRP",
            "TYR",
            "VAL",
        ]

    def SetFolders(self, *args, **kwargs):
        actual_cwd = os.getcwd()
        self.PROJECT_DIR = kwargs.get("path", os.getcwd())
        for i in args:
            self.PROJECT_DIR = i
        if self.PROJECT_DIR == ".":
            return console.print(f"Project Base Directory: {actual_cwd}")
        if actual_cwd != self.PROJECT_DIR:
            try:
                os.chdir(self.PROJECT_DIR)
                working_dir = self.PROJECT_DIR
            except Exception as err:
                working_dir = f"{actual_cwd}/{self.PROJECT_DIR}"
                os.chdir(working_dir)
        console.print(f"Project Base Directory: {working_dir}")

    def ProjectTree(self, *args, **kwargs):
        path = os.getwd() if self.PROJECT_DIR is None else self.PROJECT_DIR
        self.directory_tree(path)

    def __actual_dir_name(self, path, root=None):
        """helper function for directory tree generation
        Args:
            path ([type]): [description]
            root ([type], optional): [description]. Defaults to None.
        Returns:
            [type]: [description]
        """
        if root is not None:
            path = os.path.join(root, path)
        result = os.path.basename(path)
        if os.path.islink(path):
            realpath = os.readlink(path)
            result = f"{os.path.basename(path)} -> {realpath}"
        return result

    def directory_tree(self, startpath, depth=-1):
        """Tree view of the project directory tree"
        directory_tree(path)
        """
        table = self.__extracted_from_LoadReceptor_4(
            "bold magenta", "File Type", "Total Files"
        )
        c = Counter(
            [splitext(i)[1][1:] for i in glob(join(startpath, "**"), recursive=True)]
        )
        console.print("============Details of files====================")
        for ext, count in c.most_common():
            table.add_row(
                f"[bold green]{str(ext)}[/bold green]", f"[red]{str(count)}[/red]"
            )
        console.print(table)
        console.print("============Directory Tree====================")
        prefix = 0
        if startpath != "/":
            if startpath.endswith("/"):
                startpath = startpath[:-1]
            prefix = len(startpath)
        for root, dirs, files in os.walk(startpath):
            level = root[prefix:].count(os.sep)
            if depth > -1 and level > depth:
                continue
            indent = subindent = ""
            if level > 0:
                indent = "|   " * (level - 1) + "|-- "
            subindent = "|   " * (level) + "|-- "
            print(
                f"{indent}📂{self.__actual_dir_name(root)}/"
            )  # print dir only if symbolic link; otherwise, will be printed as root
            for d in dirs:
                if os.path.islink(os.path.join(root, d)):
                    print(f"{subindent}📃{self.__actual_dir_name(d, root=root)}")
            for f in files:
                print(f"{subindent}📃{self.__actual_dir_name(f, root=root)}")

    def __receptor_contents_print(self):
        number_of_residues = []
        number_of_membrane_molecules = []
        number_of_water_molecule = 0
        present_ions = []
        number_of_chains = []
        number_of_ligands = []
        number_of_ligands_atoms = 0
        for index, line in enumerate(self.receptor_content):
            if line.startswith("ATOM"):
                if line[17:20] in self.AA or line.split()[-1] == "PROA":
                    number_of_chains.append(line[21])
                    number_of_residues.append(line[22:26])
                elif line.split()[-1] == "MEMB":
                    number_of_membrane_molecules.append(line[22:26])
                elif line.split()[-1] == "TIP3" or line[17:20] == "HOH":
                    number_of_water_molecule += 1
                elif line.split()[-1] == "HETA":
                    number_of_ligands.append(line.split()[3])
                else:
                    present_ions.append(line[17:20])
            elif line.startswith("HETATM"):
                if line[17:20] == "HOH":
                    number_of_water_molecule += 1
                elif len(line[17:20].strip()) < 3:
                    present_ions.append(line[17:20])
                elif len(line[17:20].strip()) == 3:
                    # number_of_ligands.append(line.split()[3])
                    number_of_ligands.append(line[21])
                    number_of_ligands_atoms += 1
        if not present_ions:
            max_number_of_single_ions = 0
        else:
            max_number_of_single_ions = max(present_ions, key=present_ions.count)
        types_of_ions = set(present_ions) if present_ions else 0
        number_of_membrane_molecules = (
            number_of_membrane_molecules[-1]
            if len(number_of_membrane_molecules) > 1 and not None
            else 0
        )
        table = self.__extracted_from_LoadReceptor_4("bold blue", "Record", "Counts")
        table.add_row("[bold green]Chains:[/]", f" {len(set(number_of_chains))}")
        table.add_row("[bold green]Ligands:[/]", f"{len(set(number_of_ligands))}")
        try:
            table.add_row(
                "[bold green]Number of ligand atoms :[/]",
                f"{number_of_ligands.count(max(number_of_ligands, key=number_of_ligands.count))}",
            )
        except ValueError:
            table.add_row("[bold green]Number of ligand atoms :[/]", "0")

        table.add_row("[bold green]Protein residues:[/]", f"{number_of_residues[-1]}")

        try:
            table.add_row(
                "[bold green]Lipids molecules :[/]", f"{number_of_membrane_molecules}"
            )
        except ValueError:
            table.add_row("[bold green]Lipids molecules :[/]", "0")

        try:
            table.add_row(
                "[bold green]Water molecules :[/]", f" {number_of_water_molecule}"
            )
        except ValueError:
            table.add_row("[bold green]Water molecules :[/]", "0")
        try:
            table.add_row("[bold green]Ions:[/]", f"{len(present_ions)}")
            table.add_row("[bold green]Ion types :[/]", f"{types_of_ions}")
        except ValueError:
            table.add_row("[bold green]Ions:[/]", "0")
            table.add_row("[bold green]Ion types :[/]", "None")

        console.print(f"\nFor[bold red] {self.receptor}[/]:")
        console.print(table)

    def LoadReceptor(self, *args, **kwargs):
        for i in args:
            self.receptor = i
        try:
            _receptor = file_search(type="pdb", target=self.receptor)
            if len(_receptor) == 1:
                console.print(f"Found Receptor: [bold]{_receptor[0]}[/bold]")
                self.receptor = _receptor[0]
            elif len(_receptor) > 1:
                print(f"Found Receptors: {_receptor}")
                _receptor_number = int(input("Which receptor do you like to select: "))
                self.receptor = _receptor[_receptor_number]
                console.print(f"Select Receptor : {self.receptor}")
            else:
                print("No Receptors found in Local directory")
                _download = input(
                    f"Would you like to download the {self.receptor} from RCSB (Y/n)? "
                )
                confirm = ["yes", "y"]
                if _download in confirm:
                    download_protein_msg = get(self.receptor)
                    console.print(download_protein_msg)  # TODO path pass forward
                    self.receptor = f"./Generated/{self.receptor}.pdb"
                else:
                    console.print(
                        f"😞 No Protein: [bold red]{self.receptor} [/]to process"
                        " further.."
                    )
        except Exception as er:
            print(er)
        receptor_content = open(self.receptor, "r")
        self.receptor_content = receptor_content.readlines()
        self.__receptor_contents_print()
        # console.print(self.receptor)

    def __extracted_from_LoadReceptor_4(self, header_style, arg1, arg2):
        result = Table(show_header=True, header_style=header_style)
        result.add_column(arg1, style="dim", width=40)
        result.add_column(arg2, justify="right")
        return result

    def LoadLigand(self, *args, **kwargs):
        for arg in args:
            self.ligand = arg
        dir = kwargs.get("dir", ".")
        inf = open(f"{dir}/{self.ligand}", "rb")
        with Chem.ForwardSDMolSupplier(inf) as fsuppl:
            for mol in fsuppl:
                if mol is None:
                    continue
                console.print(f"{self.ligand} has {mol.GetNumAtoms()} atoms")
        return fsuppl

    def __call__(self):
        raise TypeError('Project must be accessed through "instance=ProjectStart()".')

    def __str__(self):
        ligand = self.ligand if self.ligand != None else "Not implemented"
        receptor = self.receptor if self.receptor != None else "Not implemented"
        return f"Project : \nProtein: {receptor}\nligand :{ligand}"
